@@ -1,19 +1,16 @@
 /**
  * Copyright 2019 Google LLC
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * <p>Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
+ * except in compliance with the License. You may obtain a copy of the License at
  *
- *     https://www.apache.org/licenses/LICENSE-2.0
+ * <p>https://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
+ * <p>Unless required by applicable law or agreed to in writing, software distributed under the
+ * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+ * express or implied. See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package com.google.api.client.test.json;
 
 import com.google.api.client.json.GenericJson;
@@ -25,6 +22,7 @@ import java.io.InputStream;
 import java.math.BigDecimal;
 import java.nio.charset.StandardCharsets;
 import junit.framework.TestCase;
+import org.junit.Assert;
 
 public abstract class AbstractJsonParserTest extends TestCase {
 
@@ -47,6 +45,29 @@ public abstract class AbstractJsonParserTest extends TestCase {
     assertEquals(Boolean.FALSE, json.get("boolValue"));
   }
 
+  public void testGetWrongType() throws IOException {
+    JsonObjectParser parser = new JsonObjectParser(newJsonFactory());
+    InputStream inputStream = new ByteArrayInputStream(TEST_JSON.getBytes(StandardCharsets.UTF_8));
+    GenericJson json = parser.parseAndClose(inputStream, StandardCharsets.UTF_8, GenericJson.class);
+    assertTrue(json.get("strValue") instanceof String);
+    assertEquals("bar", json.get("strValue"));
+    assertTrue(json.get("intValue") instanceof BigDecimal);
+    assertEquals(new BigDecimal(123), json.get("intValue"));
+    assertTrue(json.get("boolValue") instanceof Boolean);
+    assertEquals(Boolean.FALSE, json.get("boolValue"));
+  }
+
+  public void testParse_badJson() throws IOException {
+    JsonObjectParser parser = new JsonObjectParser(newJsonFactory());
+    InputStream inputStream = new ByteArrayInputStream("not json".getBytes(StandardCharsets.UTF_8));
+    try {
+      parser.parseAndClose(inputStream, StandardCharsets.UTF_8, GenericJson.class);
+      fail("Malformed JSON not detected");
+    } catch (IOException ex) {
+      Assert.assertNotNull(ex.getMessage());
+    }
+  }
+
   public void testParse_bigDecimal() throws IOException {
     JsonObjectParser parser = new JsonObjectParser(newJsonFactory());
     InputStream inputStream =
