@@ -42,6 +42,9 @@ public class HttpResponseException extends IOException {
   /** HTTP response content or {@code null} for none. */
   private final String content;
 
+  /** Number of attempts performed */
+  private final int attemptCount;
+
   /**
    * Constructor that constructs a detail message from the given HTTP response that includes the
    * status code, status message and HTTP response content.
@@ -73,6 +76,7 @@ public class HttpResponseException extends IOException {
     statusMessage = builder.statusMessage;
     headers = builder.headers;
     content = builder.content;
+    attemptCount = builder.attemptCount;
   }
 
   /**
@@ -121,6 +125,15 @@ public class HttpResponseException extends IOException {
     return content;
   }
 
+  /**
+   * Returns the attempt count
+   *
+   * @since 1.41
+   */
+  public final int getAttemptCount() {
+    return attemptCount;
+  }
+
   /**
    * Builder.
    *
@@ -145,6 +158,9 @@ public class HttpResponseException extends IOException {
     /** Detail message to use or {@code null} for none. */
     String message;
 
+    /** Number of attempts performed */
+    int attemptCount;
+
     /**
      * @param statusCode HTTP status code
      * @param statusMessage status message or {@code null}
@@ -260,6 +276,18 @@ public class HttpResponseException extends IOException {
       return this;
     }
 
+    /** Returns the request attempt count */
+    public final int getAttemptCount() {
+      return attemptCount;
+    }
+
+    /** Sets the attempt count for the related HTTP request execution. */
+    public Builder setAttemptCount(int attemptCount) {
+      Preconditions.checkArgument(attemptCount >= 0);
+      this.attemptCount = attemptCount;
+      return this;
+    }
+
     /** Returns a new instance of {@link HttpResponseException} based on this builder. */
     public HttpResponseException build() {
       return new HttpResponseException(this);
